@@ -3,6 +3,8 @@
 
 #include "lib/list.h"
 
+#include "vm/object.h"
+
 #include <stdio.h> /* for NOT_IMPLEMENTED */
 #include <pthread.h>
 
@@ -17,6 +19,11 @@ enum vm_thread_state {
 	VM_THREAD_STATE_WAITING,
 };
 
+enum thread_state {
+	THREAD_STATE_CONSISTENT,
+	THREAD_STATE_INCONSISTENT,
+};
+
 struct vm_thread {
 	pthread_mutex_t mutex;
 
@@ -26,13 +33,16 @@ struct vm_thread {
 	pthread_t posix_id;
 	enum vm_thread_state state;
 	struct list_head list_node;
+	bool interrupted;
+	struct vm_monitor *wait_mon;
+	enum thread_state thread_state;
 };
 
 struct vm_exec_env {
 	struct vm_thread *thread;
 };
 
-unsigned int vm_nr_threads_running(void);
+unsigned int vm_nr_threads(void);
 
 extern __thread struct vm_exec_env current_exec_env;
 
@@ -52,5 +62,15 @@ void vm_thread_wait_for_non_daemons(void);
 void vm_thread_set_state(struct vm_thread *thread, enum vm_thread_state state);
 struct vm_object *vm_thread_get_java_thread(struct vm_thread *thread);
 char *vm_thread_get_name(struct vm_thread *thread);
+bool vm_thread_is_interrupted(struct vm_thread *thread);
+bool vm_thread_interrupted(struct vm_thread *thread);
+void vm_thread_interrupt(struct vm_thread *thread);
+void vm_lock_thread_count(void);
+void vm_unlock_thread_count(void);
+
+extern struct list_head thread_list;
+extern pthread_mutex_t threads_mutex;
+
+#define vm_thread_for_each(this) list_for_each_entry(this, &thread_list, list_node)
 
 #endif
